MODULE module_emissions_anthropogenics
!WRF:MODEL_LAYER:CHEMICS
!
CONTAINS
!
! currently this only adds in the emissions...
! this may be done differently for different chemical mechanisms
! in the future. aerosols are already added somewhere else....
!
   subroutine add_anthropogenics(id,dtstep,dz8w,config_flags,rho_phy,chem, &
               emis_ant, emis_aircraft,                                    &
               ids,ide, jds,jde, kds,kde,                                  &
               ims,ime, jms,jme, kms,kme,                                  &
               its,ite, jts,jte, kts,kte                                   )
!----------------------------------------------------------------------
  USE module_configure
  USE module_state_description
  USE module_data_radm2
   IMPLICIT NONE

! .. Parameters ..
   TYPE(grid_config_rec_type),  INTENT(IN   )    :: config_flags

   INTEGER,      INTENT(IN   ) :: id,                                      &
                                  ids,ide, jds,jde, kds,kde,               &
                                  ims,ime, jms,jme, kms,kme,               &
                                  its,ite, jts,jte, kts,kte
      REAL,      INTENT(IN   ) ::                                          &
                             dtstep
!
   REAL, DIMENSION( ims:ime, kms:kme, jms:jme, num_chem ),                 &
         INTENT(INOUT ) ::                                   chem
!
! emissions arrays
!
!   REAL, DIMENSION( ims:ime, kms:kme, jms:jme ),                           &
   REAL, DIMENSION( ims:ime, kms:config_flags%kemit, jms:jme,num_emis_ant ),            &
         INTENT(IN ) ::                                                    &
               emis_ant        
   REAL, DIMENSION( ims:ime, kms:kme, jms:jme ),                           &
         INTENT(IN ) ::        rho_phy

!
! 
!


   REAL,  DIMENSION( ims:ime , kms:kme , jms:jme )         ,               &
          INTENT(IN   ) ::                                                 &
                                                      dz8w

! stuff for aircraft emissions

   REAL, DIMENSION( ims:ime, kms:config_flags%kemit_aircraft, jms:jme, num_emis_aircraft),      &
         OPTIONAL,INTENT(IN ) ::  emis_aircraft

    integer i,j,k
    real :: conv_rho
!--- deposition and emissions stuff


! ..
! ..
! .. Intrinsic Functions ..

         call wrf_debug(15,'add_anhropogenics')
!       
! add emissions
!
      do 100 j=jts,jte  
      do 100 i=its,ite  

      DO k=kts,min(config_flags%kemit,kte)
        conv_rho=4.828e-4/rho_phy(i,k,j)*dtstep/(dz8w(i,k,j)*60.)

#if (defined(CHEM_DBG_I) && defined(CHEM_DBG_J) && defined(CHEM_DBG_K))
       if( (i <= CHEM_DBG_I .and. i >= CHEM_DBG_I) .and. &
           (j <= CHEM_DBG_J .and. j >= CHEM_DBG_J) .and. &
           (k <= CHEM_DBG_K .and. k >= CHEM_DBG_K)  ) then
          print*
          print*,"Converted emissions for RADM2:"
          print*,"e_csl=",emis_ant(i,k,j,p_e_csl)*conv_rho
          print*,"e_iso=",emis_ant(i,k,j,p_e_iso)*conv_rho
          print*,"e_so2=",emis_ant(i,k,j,p_e_so2)*conv_rho
          print*,"e_no=",emis_ant(i,k,j,p_e_no)*conv_rho
          print*,"e_ald=",emis_ant(i,k,j,p_e_ald)*conv_rho
          print*,"e_hcho=",emis_ant(i,k,j,p_e_hcho)*conv_rho
          print*,"e_ora2=",emis_ant(i,k,j,p_e_ora2)*conv_rho
          print*,"e_nh3=",emis_ant(i,k,j,p_e_nh3)*conv_rho
          print*,"e_hc3=",emis_ant(i,k,j,p_e_hc3)*conv_rho
          print*,"e_hc5=",emis_ant(i,k,j,p_e_hc5)*conv_rho
          print*,"e_hc8=",emis_ant(i,k,j,p_e_hc8)*conv_rho
          print*,"e_eth=",emis_ant(i,k,j,p_e_eth)*conv_rho
          print*,"e_co=",emis_ant(i,k,j,p_e_co)*conv_rho
          print*,"e_ol2=",emis_ant(i,k,j,p_e_ol2)*conv_rho
          print*,"e_olt=",emis_ant(i,k,j,p_e_olt)*conv_rho
          print*,"e_oli=",emis_ant(i,k,j,p_e_oli)*conv_rho
          print*,"e_tol=",emis_ant(i,k,j,p_e_tol)*conv_rho
          print*,"e_xyl=",emis_ant(i,k,j,p_e_xyl)*conv_rho
          print*,"e_ket=",emis_ant(i,k,j,p_e_ket)*conv_rho
       end if
#endif

        chem(i,k,j,p_so2)  = chem(i,k,j,p_so2)                         &
                         +emis_ant(i,k,j,p_e_so2)*conv_rho
        chem(i,k,j,p_co)  = chem(i,k,j,p_co)                           &
                         +emis_ant(i,k,j,p_e_co)*conv_rho
        chem(i,k,j,p_no)   = chem(i,k,j,p_no)                          &
                         +emis_ant(i,k,j,p_e_no)*conv_rho
        chem(i,k,j,p_no2)   = chem(i,k,j,p_no2)                          &
                         +emis_ant(i,k,j,p_e_no2)*conv_rho
        chem(i,k,j,p_nh3)  = chem(i,k,j,p_nh3)                         &
                         +emis_ant(i,k,j,p_e_nh3)*conv_rho
        if( config_flags%chem_opt == MOZART_KPP .or. config_flags%chem_opt == MOZCART_KPP ) then
           chem(i,k,j,p_bigalk) = chem(i,k,j,p_bigalk) + emis_ant(i,k,j,p_e_bigalk)*conv_rho
           chem(i,k,j,p_bigene) = chem(i,k,j,p_bigene) + emis_ant(i,k,j,p_e_bigene)*conv_rho
           chem(i,k,j,p_c2h4)   = chem(i,k,j,p_c2h4)  + emis_ant(i,k,j,p_e_c2h4)*conv_rho
           chem(i,k,j,p_c2h5oh) = chem(i,k,j,p_c2h5oh) + emis_ant(i,k,j,p_e_c2h5oh)*conv_rho
           chem(i,k,j,p_c2h6)   = chem(i,k,j,p_c2h6) + emis_ant(i,k,j,p_e_c2h6)*conv_rho
           chem(i,k,j,p_c3h6)   = chem(i,k,j,p_c3h6) + emis_ant(i,k,j,p_e_c3h6)*conv_rho
           chem(i,k,j,p_c3h8)   = chem(i,k,j,p_c3h8) + emis_ant(i,k,j,p_e_c3h8)*conv_rho
           chem(i,k,j,p_hcho)   = chem(i,k,j,p_hcho) + emis_ant(i,k,j,p_e_ch2o)*conv_rho
           chem(i,k,j,p_ald)    = chem(i,k,j,p_ald) + emis_ant(i,k,j,p_e_ch3cho)*conv_rho
           chem(i,k,j,p_acet)   = chem(i,k,j,p_acet) + emis_ant(i,k,j,p_e_ch3coch3)*conv_rho
           chem(i,k,j,p_ch3oh)  = chem(i,k,j,p_ch3oh) + emis_ant(i,k,j,p_e_ch3oh)*conv_rho
           chem(i,k,j,p_mek)    = chem(i,k,j,p_mek) + emis_ant(i,k,j,p_e_mek)*conv_rho
           chem(i,k,j,p_tol)    = chem(i,k,j,p_tol) + emis_ant(i,k,j,p_e_toluene)*conv_rho
           chem(i,k,j,p_isopr)  = chem(i,k,j,p_isopr) + emis_ant(i,k,j,p_e_isop)*conv_rho
           chem(i,k,j,p_c10h16) = chem(i,k,j,p_c10h16) + emis_ant(i,k,j,p_e_c10h16)*conv_rho
           chem(i,k,j,p_no2)    = chem(i,k,j,p_no2) + emis_ant(i,k,j,p_e_no2)*conv_rho
          else
        chem(i,k,j,p_csl)  =  chem(i,k,j,p_csl)                        &
                         +emis_ant(i,k,j,p_e_csl)*conv_rho
        chem(i,k,j,p_iso)  = chem(i,k,j,p_iso)                         &
                         +emis_ant(i,k,j,p_e_iso)*conv_rho
        chem(i,k,j,p_ald)  = chem(i,k,j,p_ald)                         &
                         +emis_ant(i,k,j,p_e_ald)*conv_rho
        chem(i,k,j,p_hcho) = chem(i,k,j,p_hcho)                        &
                         +emis_ant(i,k,j,p_e_hcho)*conv_rho
        chem(i,k,j,p_ora2)  = chem(i,k,j,p_ora2)                       &
                         +emis_ant(i,k,j,p_e_ora2)*conv_rho
        chem(i,k,j,p_hc3)  = chem(i,k,j,p_hc3)                         &
                         +emis_ant(i,k,j,p_e_hc3)*conv_rho
        chem(i,k,j,p_hc5)  = chem(i,k,j,p_hc5)                         &
                         +emis_ant(i,k,j,p_e_hc5)*conv_rho
        chem(i,k,j,p_hc8)  = chem(i,k,j,p_hc8)                         &
                         +emis_ant(i,k,j,p_e_hc8)*conv_rho
        chem(i,k,j,p_eth)  = chem(i,k,j,p_eth)                         &
                         +emis_ant(i,k,j,p_e_eth)*conv_rho
        if(p_ol2.gt.1)chem(i,k,j,p_ol2)  = chem(i,k,j,p_ol2)           &
                         +emis_ant(i,k,j,p_e_ol2)*conv_rho
        if(p_ete.gt.1)chem(i,k,j,p_ete)  = chem(i,k,j,p_ete)           &
                         +emis_ant(i,k,j,p_e_ol2)*conv_rho
        chem(i,k,j,p_olt)  = chem(i,k,j,p_olt)                         &
                         +emis_ant(i,k,j,p_e_olt)*conv_rho
        chem(i,k,j,p_oli)  = chem(i,k,j,p_oli)                         &
                         +emis_ant(i,k,j,p_e_oli)*conv_rho
        chem(i,k,j,p_tol)  = chem(i,k,j,p_tol)                         &
                         +emis_ant(i,k,j,p_e_tol)*conv_rho
        chem(i,k,j,p_xyl)  = chem(i,k,j,p_xyl)                         &      
                         +emis_ant(i,k,j,p_e_xyl)*conv_rho
        chem(i,k,j,p_ket)  =  chem(i,k,j,p_ket)                        &     
                         +emis_ant(i,k,j,p_e_ket)*conv_rho
       endif
      END DO                                                          
 100  continue

! add aircraft emissions
   if (config_flags%aircraft_emiss_opt == 1 ) then
      do j=jts,jte  
      do k=kts,min(config_flags%kemit_aircraft,kte)
      do i=its,ite  
         conv_rho=4.828e-4/rho_phy(i,k,j)*dtstep/(dz8w(i,k,j)*60.)
         chem(i,k,j,p_no)  = chem(i,k,j,p_no)  + emis_aircraft(i,k,j,p_eac_no) *conv_rho
         chem(i,k,j,p_co)  = chem(i,k,j,p_co)  + emis_aircraft(i,k,j,p_eac_co) *conv_rho
         chem(i,k,j,p_so2) = chem(i,k,j,p_so2) + emis_aircraft(i,k,j,p_eac_so2)*conv_rho
         chem(i,k,j,p_ch4) = chem(i,k,j,p_ch4) + emis_aircraft(i,k,j,p_eac_ch4)*conv_rho
      enddo
      enddo
      enddo
   endif 

    END subroutine add_anthropogenics
!
!
    subroutine add_biogenics(id,dtstep,dz8w,config_flags,rho_phy,chem, &
         e_bio,ne_area,                                                &
         ebio_iso,ebio_oli,ebio_api,ebio_lim,ebio_xyl,                 &
         ebio_hc3,ebio_ete,ebio_olt,ebio_ket,ebio_ald,                 &
         ebio_hcho,ebio_eth,ebio_ora2,ebio_co,ebio_nr,ebio_no,         &
         ids,ide, jds,jde, kds,kde,                                    &
         ims,ime, jms,jme, kms,kme,                                    &
         its,ite, jts,jte, kts,kte                                     )
  USE module_configure
  USE module_state_description                                  
  USE module_data_radm2                               
  USE module_aerosols_sorgam 
   IMPLICIT NONE             
   INTEGER,      INTENT(IN   ) :: id,ne_area,                              &
                                  ids,ide, jds,jde, kds,kde,               &
                                  ims,ime, jms,jme, kms,kme,               &
                                  its,ite, jts,jte, kts,kte
   REAL,         INTENT(IN   ) ::                                          &
                             dtstep
   REAL, DIMENSION( ims:ime, kms:kme, jms:jme, num_chem ),                 &
         INTENT(INOUT ) ::                                   chem
   REAL, DIMENSION( ims:ime, jms:jme,ne_area ),                            &
         INTENT(IN ) ::                                                    &
                 e_bio
   REAL, DIMENSION( ims:ime, jms:jme ),                                    &
         INTENT(IN ) ::                                                    &
         ebio_iso,ebio_oli,ebio_api,ebio_lim,ebio_xyl,                     &
         ebio_hc3,ebio_ete,ebio_olt,ebio_ket,ebio_ald,                     &
         ebio_hcho,ebio_eth,ebio_ora2,ebio_co,ebio_nr,ebio_no

!
! 
!

   REAL,  DIMENSION( ims:ime , kms:kme , jms:jme )         ,               &
          INTENT(IN   ) ::                                                 &
                                          rho_phy,dz8w
    integer i,j,k,n
    real :: conv_rho
!--- deposition and emissions stuff
! .. Parameters ..       
   TYPE(grid_config_rec_type),  INTENT(IN   )    :: config_flags     
!  return
   bioem_select: SELECT CASE(config_flags%bio_emiss_opt)
     CASE (GUNTHER1)
     CALL wrf_debug(15,'adding biogenic emissions: Gunther1')
!     DO n = 1, numgas-2
      do 100 j=jts,jte  
      do 100 i=its,ite  
        conv_rho=dtstep/(dz8w(i,kts,j)*60.)
        chem(i,kts,j,p_iso)=chem(i,kts,j,p_iso)+ &
                  e_bio(i,j,p_iso-1)*conv_rho
        chem(i,kts,j,p_oli)=chem(i,kts,j,p_oli)+ &
                  e_bio(i,j,p_oli-1)*conv_rho
        chem(i,kts,j,p_xyl)=chem(i,kts,j,p_xyl)+ &
                  e_bio(i,j,p_xyl-1)*conv_rho
        chem(i,kts,j,p_hc3)=chem(i,kts,j,p_hc3)+ &
                  e_bio(i,j,p_hc3-1)*conv_rho
        chem(i,kts,j,p_olt)=chem(i,kts,j,p_olt)+ &
                  e_bio(i,j,p_olt-1)*conv_rho
        chem(i,kts,j,p_ket)=chem(i,kts,j,p_ket)+ &
                  e_bio(i,j,p_ket-1)*conv_rho
        chem(i,kts,j,p_ald)=chem(i,kts,j,p_ald)+ &
                  e_bio(i,j,p_ald-1)*conv_rho
        chem(i,kts,j,p_hcho)=chem(i,kts,j,p_hcho)+ &
                  e_bio(i,j,p_hcho-1)*conv_rho
        chem(i,kts,j,p_eth)=chem(i,kts,j,p_eth)+ &
                  e_bio(i,j,p_eth-1)*conv_rho
        chem(i,kts,j,p_ora2)=chem(i,kts,j,p_ora2)+ &
                  e_bio(i,j,p_ora2-1)*conv_rho
        chem(i,kts,j,p_co)=chem(i,kts,j,p_co)+ &
                  e_bio(i,j,p_co-1)*conv_rho
        chem(i,kts,j,p_no)=chem(i,kts,j,p_no)+ &
                  e_bio(i,j,p_no-1)*conv_rho
!
! RADM only
!
        if(p_ol2.gt.1)chem(i,kts,j,p_ol2)=chem(i,kts,j,p_ol2)+ &
                  e_bio(i,j,p_ol2-1)*conv_rho
 100  continue
!     enddo
     CASE (BEIS313)
     CALL wrf_debug(100,'adding biogenic emissions: beis3.1.3')
      do j=jts,jte  
      do i=its,ite  
        conv_rho=4.828e-4/rho_phy(i,kts,j)*dtstep/(dz8w(i,kts,j)*60.)
        chem(i,kts,j,p_iso)=chem(i,kts,j,p_iso)+ &
                  ebio_iso(i,j)*conv_rho
        chem(i,kts,j,p_oli)=chem(i,kts,j,p_oli)+ &
                  ebio_oli(i,j)*conv_rho
        chem(i,kts,j,p_xyl)=chem(i,kts,j,p_xyl)+ &
                  ebio_xyl(i,j)*conv_rho
        chem(i,kts,j,p_hc3)=chem(i,kts,j,p_hc3)+ &
                  ebio_hc3(i,j)*conv_rho
        chem(i,kts,j,p_olt)=chem(i,kts,j,p_olt)+ &
                  ebio_olt(i,j)*conv_rho
        chem(i,kts,j,p_ket)=chem(i,kts,j,p_ket)+ &
                  ebio_ket(i,j)*conv_rho
        chem(i,kts,j,p_ald)=chem(i,kts,j,p_ald)+ &
                  ebio_ald(i,j)*conv_rho
        chem(i,kts,j,p_hcho)=chem(i,kts,j,p_hcho)+ &
                  ebio_hcho(i,j)*conv_rho
        chem(i,kts,j,p_eth)=chem(i,kts,j,p_eth)+ &
                  ebio_eth(i,j)*conv_rho
        chem(i,kts,j,p_ora2)=chem(i,kts,j,p_ora2)+ &
                  ebio_ora2(i,j)*conv_rho
        chem(i,kts,j,p_co)=chem(i,kts,j,p_co)+ &
                  ebio_co(i,j)*conv_rho
        chem(i,kts,j,p_no)=chem(i,kts,j,p_no)+ &
                  ebio_no(i,j)*conv_rho
!
! RADM only
!
        if(p_ol2.gt.1)chem(i,kts,j,p_ol2)=chem(i,kts,j,p_ol2)+ &
                  ebio_ete(i,j)*conv_rho
!
! RACM only
!
        if(p_api.gt.1)chem(i,kts,j,p_api)=chem(i,kts,j,p_api)+ &
                  ebio_api(i,j)*conv_rho
        if(p_lim.gt.1)chem(i,kts,j,p_lim)=chem(i,kts,j,p_lim)+ &
                  ebio_lim(i,j)*conv_rho
        if(p_ete.gt.1)chem(i,kts,j,p_ete)=chem(i,kts,j,p_ete)+ &
                  ebio_ete(i,j)*conv_rho
      enddo
      enddo
     CASE DEFAULT

   END SELECT bioem_select
    END subroutine add_biogenics


END MODULE module_emissions_anthropogenics
