!**********************************************************************************  
! This computer software was prepared by Battelle Memorial Institute, hereinafter
! the Contractor, under Contract No. DE-AC05-76RL0 1830 with the Department of 
! Energy (DOE). NEITHER THE GOVERNMENT NOR THE CONTRACTOR MAKES ANY WARRANTY,
! EXPRESS OR IMPLIED, OR ASSUMES ANY LIABILITY FOR THE USE OF THIS SOFTWARE.
!
! MOSAIC module: see module_mosaic_driver.F for references and terms of use
!**********************************************************************************  

!----------------------------------------------------------------------
! This module contains interface wrapper routines to couple the aerosol
! modules with mixactivate in the physics directory. Due to compiling
! dependencies, these cannot be placed in module_mixactivate.
!----------------------------------------------------------------------

MODULE module_mixactivate_wrappers

CONTAINS

!----------------------------------------------------------------------
!----------------------------------------------------------------------
      subroutine mosaic_mixactivate (                        &
           id, ktau, dtstep, config_flags, idrydep_onoff,    &
           rho_phy, t_phy, w, cldfra, cldfra_old,            &
           ddvel, z, dz8w, p_at_w, t_at_w, exch_h,           &
           qv, qc, qi, qndrop3d, f_qc, f_qi, chem,           &
	       ccn1, ccn2, ccn3, ccn4, ccn5, ccn6, nsource,      &
           ids,ide, jds,jde, kds,kde,                        &
           ims,ime, jms,jme, kms,kme,                        &
           its,ite, jts,jte, kts,kte,                        &
               k_ff_a01,k_ff_a02,k_ff_a03,k_ff_a04,                       &
               k_ff_a05,k_ff_a06,k_ff_a07,k_ff_a08,                       &

               k_bb_a01,k_bb_a02,k_bb_a03,k_bb_a04,                       &
               k_bb_a05,k_bb_a06,k_bb_a07,k_bb_a08,                       &

               k_ff_cw01,k_ff_cw02,k_ff_cw03,k_ff_cw04,                   &
               k_ff_cw05,k_ff_cw06,k_ff_cw07,k_ff_cw08,                   &

               k_bb_cw01,k_bb_cw02,k_bb_cw03,k_bb_cw04,                   &
               k_bb_cw05,k_bb_cw06,k_bb_cw07,k_bb_cw08                   &
                         )

    USE module_configure, only: grid_config_rec_type
	use module_state_description, only:  num_chem
	use module_data_mosaic_asect
	use module_mixactivate, only: mixactivate

! wrapper to call mixactivate for mosaic description of aerosol

	implicit none

!   subr arguments
	integer, intent(in) ::               &
         id, ktau,                       &
         ids, ide, jds, jde, kds, kde,   &
         ims, ime, jms, jme, kms, kme,   &
         its, ite, jts, jte, kts, kte,   &
         idrydep_onoff

	real, intent(in) :: dtstep

	real, intent(in),   &
		dimension( ims:ime, kms:kme, jms:jme ) :: &
		rho_phy, t_phy, w,   &
		z, dz8w, p_at_w, t_at_w, exch_h

	real, intent(inout),   &
		dimension( ims:ime, kms:kme, jms:jme ) :: cldfra, cldfra_old

	real, intent(in),   &
		dimension( its:ite, jts:jte, num_chem ) :: ddvel

	real, intent(in),   &
		dimension( ims:ime, kms:kme, jms:jme ) :: &
		qv, qc, qi

    LOGICAL, intent(in) :: f_qc, f_qi

	real, intent(inout),   &
		dimension( ims:ime, kms:kme, jms:jme ) :: &
		qndrop3d

	real, intent(inout),   &
		dimension( ims:ime, kms:kme, jms:jme, 1:num_chem ) :: &
		chem
      real, intent(out), dimension(ims:ime,kms:kme,jms:jme) :: nsource,&
	     ccn1,ccn2,ccn3,ccn4,ccn5,ccn6  ! number conc of aerosols activated at supersat

	type(grid_config_rec_type), intent(in) :: config_flags
! local vars
	real qsrflx(ims:ime, jms:jme, num_chem) ! dry deposition flux of aerosol
	real(kind=8) sumhygro,sumvol
	integer i,j,k,l,m,n
	real hygro( its:ite, kts:kte, jts:jte, maxd_asize, maxd_atype ) ! bulk
        real(kind=8)  tmpotoc,tmp_cw

   REAL, DIMENSION( ims:ime , kms:kme, jms:jme ), &
              INTENT(INOUT)    ::  k_ff_a01,k_ff_a02,k_ff_a03,k_ff_a04,  &
                                   k_ff_a05,k_ff_a06,k_ff_a07,k_ff_a08,  &

                                   k_bb_a01,k_bb_a02,k_bb_a03,k_bb_a04,                       &
                                   k_bb_a05,k_bb_a06,k_bb_a07,k_bb_a08,                       &

                                   k_ff_cw01,k_ff_cw02,k_ff_cw03,k_ff_cw04,                   &
                                   k_ff_cw05,k_ff_cw06,k_ff_cw07,k_ff_cw08,                   &

                                   k_bb_cw01,k_bb_cw02,k_bb_cw03,k_bb_cw04,                   &
                                   k_bb_cw05,k_bb_cw06,k_bb_cw07,k_bb_cw08


   INTEGER :: imax,jmax,kmax

   imax = min(ite,ide-1)
   jmax = min(jte,jde-1)
   kmax = kte

    k_ff_a01(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_a02(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_a03(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_a04(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_a05(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_a06(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_a07(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_a08(its:imax,kts:kmax,jts:jmax)    = 0.



    k_bb_a01(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_a02(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_a03(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_a04(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_a05(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_a06(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_a07(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_a08(its:imax,kts:kmax,jts:jmax)    = 0.



    k_ff_cw01(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_cw02(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_cw03(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_cw04(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_cw05(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_cw06(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_cw07(its:imax,kts:kmax,jts:jmax)    = 0.
    k_ff_cw08(its:imax,kts:kmax,jts:jmax)    = 0.


    k_bb_cw01(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_cw02(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_cw03(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_cw04(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_cw05(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_cw06(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_cw07(its:imax,kts:kmax,jts:jmax)    = 0.
    k_bb_cw08(its:imax,kts:kmax,jts:jmax)    = 0.


!      write (0,*) 'Within mixactivate by Manish',ncomp_aer(1)

       tmpotoc=0.0
! calculate volume-weighted bulk hygroscopicity for each type and size
      do 100 j=jts,jte
      do 100 k=kts,kte
      do 100 i=its,ite
       do n=1,ntype_aer
       do m=1,nsize_aer(n)
	        sumhygro=0.0d0
	        sumvol=0.0d0
	       do l=1,ncomp_aer(n)
!c$$$!      Coded by Manish Shrivastava on 05/11/2011
!c$$$!      Overwrite hygroscopicites of SOA from biomass and fossil fuels based on O:C ratio
!c$$$!       Overwrite hygroscopicities for opcg1_b_c and opcg1_b_o
!c$$$
!c$$$!       write (0,*) 'ncomploop'
!c$$$
!c$$$                 if(name_aer(l,n).eq.'opcg1_b_o'.or.name_aer(l,n).eq.'opcg1_b_c') then !assign same hygro for carbon and oxygen parts
!c$$$
!c$$$!              write(0,*) 'Identified lptr biomass arrays by Manish'
!c$$$                         
!c$$$
!c$$$                   if(chem(i,k,j,lptr_opcg1_b_c_aer(m,n,ai_phase)).ge.1d-2) then
!c$$$!        Overwrite hygroscopicity only if concentration of opcg1_b_c_aer is significant
!c$$$ 
!c$$$                tmpotoc=(chem(i,k,j,lptr_opcg1_b_o_aer(m,n,ai_phase))/ &
!c$$$                                (chem(i,k,j,lptr_opcg1_b_c_aer(m,n,ai_phase))/1.17d0)*12d0/16d0) 
!c$$$
!c$$$
!c$$$                 if (tmpotoc.gt.0.2d0.and.tmpotoc.lt.0.8d0) then
!c$$$                 hygro_aer(l,n)= -2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$
!c$$$                 if (m.eq.1) k_bb_a01(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.2) k_bb_a02(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.3) k_bb_a03(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.4) k_bb_a04(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.5) k_bb_a05(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.6) k_bb_a06(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.7) k_bb_a07(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.8) k_bb_a08(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$
!c$$$ 
!c$$$
!c$$$                 elseif (tmpotoc.ge.0.8d0) then
!c$$$                  hygro_aer(l,n)= 0.2d0
!c$$$
!c$$$                  if (m.eq.1) k_bb_a01(i,k,j)=0.2d0
!c$$$                  if (m.eq.2) k_bb_a02(i,k,j)=0.2d0
!c$$$                  if (m.eq.3) k_bb_a03(i,k,j)=0.2d0
!c$$$                  if (m.eq.4) k_bb_a04(i,k,j)=0.2d0
!c$$$                  if (m.eq.5) k_bb_a05(i,k,j)=0.2d0
!c$$$                  if (m.eq.6) k_bb_a06(i,k,j)=0.2d0
!c$$$                  if (m.eq.7) k_bb_a07(i,k,j)=0.2d0
!c$$$                  if (m.eq.8) k_bb_a08(i,k,j)=0.2d0
!c$$$
!c$$$
!c$$$
!c$$$                 elseif (tmpotoc.lt.0.2d0) then
!c$$$                  hygro_aer(l,n)= 1.0d-6
!c$$$
!c$$$                  if (m.eq.1) k_bb_a01(i,k,j)=1.0d-6
!c$$$                  if (m.eq.2) k_bb_a02(i,k,j)=1.0d-6
!c$$$                  if (m.eq.3) k_bb_a03(i,k,j)=1.0d-6
!c$$$                  if (m.eq.4) k_bb_a04(i,k,j)=1.0d-6
!c$$$                  if (m.eq.5) k_bb_a05(i,k,j)=1.0d-6
!c$$$                  if (m.eq.6) k_bb_a06(i,k,j)=1.0d-6
!c$$$                  if (m.eq.7) k_bb_a07(i,k,j)=1.0d-6
!c$$$                  if (m.eq.8) k_bb_a08(i,k,j)=1.0d-6
!c$$$                 
!c$$$
!c$$$                 endif
!c$$$
!c$$$                   endif
!c$$$
!c$$$! Now calculate O to C ratios for cloud borne species
!c$$$! Note the O to C for cloud borne species do not affect aerosol activation so they do not affect hygro_aer(l,n)
!c$$$! Cloud borne O to C ratios are output for diagnotics purposes
!c$$$
!c$$$                   if(chem(i,k,j,lptr_opcg1_b_c_aer(m,n,cw_phase)).ge.1d-2) then
!c$$$!        Overwrite hygroscopicity only if concentration of opcg1_b_c_aer is significant
!c$$$
!c$$$                tmpotoc=(chem(i,k,j,lptr_opcg1_b_o_aer(m,n,cw_phase))/ &
!c$$$                                (chem(i,k,j,lptr_opcg1_b_c_aer(m,n,cw_phase))/1.17d0)*12d0/16d0)
!c$$$
!c$$$                 if (tmpotoc.gt.0.2d0.and.tmpotoc.lt.0.8d0) then
!c$$$
!c$$$                 if (m.eq.1) k_bb_cw01(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.2) k_bb_cw02(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.3) k_bb_cw03(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.4) k_bb_cw04(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.5) k_bb_cw05(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.6) k_bb_cw06(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.7) k_bb_cw07(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.8) k_bb_cw08(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$
!c$$$
!c$$$                 elseif (tmpotoc.ge.0.8d0) then
!c$$$
!c$$$                  if (m.eq.1) k_bb_cw01(i,k,j)=0.2d0
!c$$$                  if (m.eq.2) k_bb_cw02(i,k,j)=0.2d0
!c$$$                  if (m.eq.3) k_bb_cw03(i,k,j)=0.2d0
!c$$$                  if (m.eq.4) k_bb_cw04(i,k,j)=0.2d0
!c$$$                  if (m.eq.5) k_bb_cw05(i,k,j)=0.2d0
!c$$$                  if (m.eq.6) k_bb_cw06(i,k,j)=0.2d0
!c$$$                  if (m.eq.7) k_bb_cw07(i,k,j)=0.2d0
!c$$$                  if (m.eq.8) k_bb_cw08(i,k,j)=0.2d0
!c$$$
!c$$$
!c$$$
!c$$$                 elseif (tmpotoc.lt.0.2d0) then
!c$$$
!c$$$                  if (m.eq.1) k_bb_cw01(i,k,j)=1.0d-6
!c$$$                  if (m.eq.2) k_bb_cw02(i,k,j)=1.0d-6
!c$$$                  if (m.eq.3) k_bb_cw03(i,k,j)=1.0d-6
!c$$$                  if (m.eq.4) k_bb_cw04(i,k,j)=1.0d-6
!c$$$                  if (m.eq.5) k_bb_cw05(i,k,j)=1.0d-6
!c$$$                  if (m.eq.6) k_bb_cw06(i,k,j)=1.0d-6
!c$$$                  if (m.eq.7) k_bb_cw07(i,k,j)=1.0d-6
!c$$$                  if (m.eq.8) k_bb_cw08(i,k,j)=1.0d-6
!c$$$
!c$$$
!c$$$                 endif
!c$$$
!c$$$                   endif
!c$$$
!c$$$
!c$$$
!c$$$                 endif
!c$$$
!c$$$
!c$$$!       Overwrite hygroscopicities for opcg1_f_c and opcg1_f_o
!c$$$                 if(name_aer(l,n).eq.'opcg1_f_o'.or.name_aer(l,n).eq.'opcg1_f_c') then !assign same hygro for carbon and oxygen parts
!c$$$
!c$$$!              write(0,*) 'Identified lptr fossil arrays by Manish'
!c$$$
!c$$$                   if(chem(i,k,j,lptr_opcg1_f_c_aer(m,n,ai_phase)).ge.1d-2) then
!c$$$!        Overwrite hygroscopicity only if concentration of opcg1_f_c_aer is significant
!c$$$
!c$$$                 tmpotoc= (chem(i,k,j,lptr_opcg1_f_o_aer(m,n,ai_phase))/ &
!c$$$                                (chem(i,k,j,lptr_opcg1_f_c_aer(m,n,ai_phase))/1.17d0)*12d0/16d0)
!c$$$
!c$$$                   if (tmpotoc.gt.0.2d0.and.tmpotoc.lt.0.8d0) then
!c$$$!         if(k.eq.1.and.m.eq.1) &
!c$$$!       write(0,*) 'Manish fossil tmpotoc between 0.2 and 0.8',  tmpotoc
!c$$$
!c$$$
!c$$$
!c$$$                 hygro_aer(l,n)= -2.654357d0*tmpotoc**3d0 + 3.904979d0*tmpotoc**2d0 &
!c$$$                                   -1.350333d0*tmpotoc + 0.136888d0
!c$$$
!c$$$                 
!c$$$
!c$$$                 if (m.eq.1) k_ff_a01(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$
!c$$$!                 if (m.eq.1) write(0,*) 'k_ff_a01', tmpotoc,k_ff_a01(i,k,j),n,ai_phase
!c$$$
!c$$$                 if (m.eq.2) k_ff_a02(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.3) k_ff_a03(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.4) k_ff_a04(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.5) k_ff_a05(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$
!c$$$                 if (m.eq.6) k_ff_a06(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.7) k_ff_a07(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.8) k_ff_a08(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$
!c$$$
!c$$$
!c$$$!          if(k.eq.1.and.m.eq.1) &
!c$$$!            write(0,*) 'Manish k_ff_a01 is', k_ff_a01(i,k,j)
!c$$$                 
!c$$$                 elseif (tmpotoc.ge.0.8d0) then
!c$$$                  hygro_aer(l,n)= 0.20d0
!c$$$                  if (m.eq.1) k_ff_a01(i,k,j)=0.2d0
!c$$$                  if (m.eq.2) k_ff_a02(i,k,j)=0.2d0
!c$$$                  if (m.eq.3) k_ff_a03(i,k,j)=0.2d0
!c$$$                  if (m.eq.4) k_ff_a04(i,k,j)=0.2d0
!c$$$                  if (m.eq.5) k_ff_a05(i,k,j)=0.2d0
!c$$$                  if (m.eq.6) k_ff_a06(i,k,j)=0.2d0
!c$$$                  if (m.eq.7) k_ff_a07(i,k,j)=0.2d0
!c$$$                  if (m.eq.8) k_ff_a08(i,k,j)=0.2d0
!c$$$
!c$$$
!c$$$                 
!c$$$                 elseif (tmpotoc.lt.0.2d0) then
!c$$$                  hygro_aer(l,n)= 1.0d-6
!c$$$
!c$$$                  if (m.eq.1) k_ff_a01(i,k,j)=1.0d-6
!c$$$                  if (m.eq.2) k_ff_a02(i,k,j)=1.0d-6
!c$$$                  if (m.eq.3) k_ff_a03(i,k,j)=1.0d-6
!c$$$                  if (m.eq.4) k_ff_a04(i,k,j)=1.0d-6
!c$$$                  if (m.eq.5) k_ff_a05(i,k,j)=1.0d-6
!c$$$                  if (m.eq.6) k_ff_a06(i,k,j)=1.0d-6
!c$$$                  if (m.eq.7) k_ff_a07(i,k,j)=1.0d-6
!c$$$                  if (m.eq.8) k_ff_a08(i,k,j)=1.0d-6
!c$$$
!c$$$
!c$$$                 endif
!c$$$ 
!c$$$                   endif
!c$$$
!c$$$
!c$$$! Now calculate O to C ratios for cloud borne species
!c$$$                   if(chem(i,k,j,lptr_opcg1_f_c_aer(m,n,cw_phase)).ge.1d-2) then
!c$$$!        Overwrite hygroscopicity only if concentration of opcg1_f_c_aer is significant
!c$$$
!c$$$                tmpotoc=(chem(i,k,j,lptr_opcg1_f_o_aer(m,n,cw_phase))/ &
!c$$$                                (chem(i,k,j,lptr_opcg1_f_c_aer(m,n,cw_phase))/1.17d0)*12d0/16d0)
!c$$$
!c$$$                 if (tmpotoc.gt.0.2d0.and.tmpotoc.lt.0.8d0) then
!c$$$
!c$$$                 if (m.eq.1) k_ff_cw01(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.2) k_ff_cw02(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.3) k_ff_cw03(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.4) k_ff_cw04(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.5) k_ff_cw05(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.6) k_ff_cw06(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.7) k_ff_cw07(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$                 if (m.eq.8) k_ff_cw08(i,k,j)=-2.654357d0*tmpotoc**3d0 +3.904979d0*tmpotoc**2d0 &
!c$$$                                    -1.350333d0*tmpotoc + 0.136888d0
!c$$$
!c$$$
!c$$$
!c$$$                 elseif (tmpotoc.ge.0.8d0) then
!c$$$
!c$$$                  if (m.eq.1) k_ff_cw01(i,k,j)=0.2d0
!c$$$                  if (m.eq.2) k_ff_cw02(i,k,j)=0.2d0
!c$$$                  if (m.eq.3) k_ff_cw03(i,k,j)=0.2d0
!c$$$                  if (m.eq.4) k_ff_cw04(i,k,j)=0.2d0
!c$$$                  if (m.eq.5) k_ff_cw05(i,k,j)=0.2d0
!c$$$                  if (m.eq.6) k_ff_cw06(i,k,j)=0.2d0
!c$$$                  if (m.eq.7) k_ff_cw07(i,k,j)=0.2d0
!c$$$                  if (m.eq.8) k_ff_cw08(i,k,j)=0.2d0
!c$$$
!c$$$
!c$$$
!c$$$                 elseif (tmpotoc.lt.0.2d0) then
!c$$$
!c$$$                  if (m.eq.1) k_ff_cw01(i,k,j)=1.0d-6
!c$$$                  if (m.eq.2) k_ff_cw02(i,k,j)=1.0d-6
!c$$$                  if (m.eq.3) k_ff_cw03(i,k,j)=1.0d-6
!c$$$                  if (m.eq.4) k_ff_cw04(i,k,j)=1.0d-6
!c$$$                  if (m.eq.5) k_ff_cw05(i,k,j)=1.0d-6
!c$$$                  if (m.eq.6) k_ff_cw06(i,k,j)=1.0d-6
!c$$$                  if (m.eq.7) k_ff_cw07(i,k,j)=1.0d-6
!c$$$                  if (m.eq.8) k_ff_cw08(i,k,j)=1.0d-6
!c$$$
!c$$$
!c$$$                 endif
!c$$$
!c$$$                   endif
!c$$$
!c$$$
!c$$$
!c$$$                 endif

 
	          sumhygro = sumhygro+hygro_aer(l,n)*   &
                   chem(i,k,j,massptr_aer(l,m,n,ai_phase))/dens_aer(l,n)
	          sumvol = sumvol+chem(i,k,j,massptr_aer(l,m,n,ai_phase))/dens_aer(l,n)
	       end do ! comp
           hygro(i,k,j,m,n)=sumhygro/sumvol
! Finished calculation hygroscopicity for a given size bin
! Now sumhygro and sumvol are reinitialized to 0 for the next size bin calculation
	end do ! size
	end do ! type
  100 continue

! check arguments of mixactivate for consistency between send, receive
! 06-nov-2005 rce - id & ktau added to arg list
      call mixactivate(  msectional, &
           chem, num_chem, qv, qc, qi, qndrop3d,   &
           t_phy, w, ddvel, idrydep_onoff,  &
           maxd_acomp, maxd_asize, maxd_atype, maxd_aphase,   &
           ncomp_aer, nsize_aer, ntype_aer, nphase_aer,  &
           numptr_aer, massptr_aer, dlo_sect, dhi_sect, sigmag_aer, dcen_sect,  &
           dens_aer, mw_aer,           &
           waterptr_aer, hygro,  ai_phase, cw_phase,                &
           ids,ide, jds,jde, kds,kde,                            &
           ims,ime, jms,jme, kms,kme,                            &
           its,ite, jts,jte, kts,kte,                            &
           rho_phy, z, dz8w, p_at_w, t_at_w, exch_h,      &
           cldfra, cldfra_old, qsrflx, &
	       ccn1, ccn2, ccn3, ccn4, ccn5, ccn6, nsource,       &
	       id, ktau, dtstep, &
           f_qc, f_qi               )

      end subroutine mosaic_mixactivate


!----------------------------------------------------------------------
!----------------------------------------------------------------------

      subroutine mosaic_mixactivate_init(                    &
           config_flags, chem, scalar,                       &
           chem_in_opt,                                      &
           ims,ime, jms,jme, kms,kme,                        &
           its,ite, jts,jte, kts,kte                         )

      USE module_configure, only: grid_config_rec_type
      use module_state_description, only:  num_chem, num_scalar, p_qndrop
      use module_data_mosaic_asect

	implicit none

! subr arguments
      type(grid_config_rec_type), intent(in) :: config_flags

      integer, intent(in) ::               &
           ims, ime, jms, jme, kms, kme,   &
           its, ite, jts, jte, kts, kte

   INTEGER,      INTENT(IN   ) :: chem_in_opt


      real, intent(inout),   &
           dimension( ims:ime, kms:kme, jms:jme, 1:num_chem ) :: &
           chem

      real, intent(inout),   &
           dimension( ims:ime, kms:kme, jms:jme, 1:num_scalar ) :: &
           scalar

      integer :: i, j, k, m, n, l

      do j=jts,jte
         do k=kts,kte
            do i=its,ite
               scalar(i,k,j,p_qndrop) = 0.               
            end do
         end do
      end do

      if( cw_phase > 0 ) then   !sanity check in case using prognostic
                                !drop number without aq. chemistry

        if(config_flags%chem_in_opt.ne.1) then

         do n=1,ntype_aer
         do m=1,nsize_aer(n)
            chem(its:ite,kts:kte,jts:jte,numptr_aer(m,n,cw_phase)) = 0.
            do l=1,ncomp_aer(n)
               chem(its:ite,kts:kte,jts:jte,massptr_aer(l,m,n,cw_phase)) = 0.
            end do              ! comp
         end do                 ! size
         end do                 ! type


       elseif (config_flags%chem_in_opt == 1) then
     
         do n=1,ntype_aer
         do m=1,nsize_aer(n)
            chem(its:ite,kts:kte,jts:jte,numptr_aer(m,n,cw_phase)) = 0.


            do l=1,ncomp_aer(n)

                     if( ai_phase > 0 ) then

! add cloud borne aerosol mass to interstitial aerosol mass

                chem(its:ite,kts:kte,jts:jte,massptr_aer(l,m,n,ai_phase))= &
                chem(its:ite,kts:kte,jts:jte,massptr_aer(l,m,n,ai_phase)) + &
                chem(its:ite,kts:kte,jts:jte,massptr_aer(l,m,n,cw_phase))

                     endif ! for ai_phase>0

               chem(its:ite,kts:kte,jts:jte,massptr_aer(l,m,n,cw_phase)) = 0.
            end do              ! comp
         end do                 ! size
         end do                 ! type
        
      endif ! for chem_in_opt

      end if !for cw_phase>0

      end subroutine mosaic_mixactivate_init


!----------------------------------------------------------------------
!----------------------------------------------------------------------


      subroutine sorgam_mixactivate (                        &
           id, ktau, dtstep, config_flags, idrydep_onoff,    &
           rho_phy, t_phy, w, cldfra, cldfra_old,            &
           ddvel, z, dz8w, p_at_w, t_at_w, exch_h,           &
           qv, qc, qi, qndrop3d, f_qc, f_qi, chem,           &
	       ccn1, ccn2, ccn3, ccn4, ccn5, ccn6, nsource,      &
           ids,ide, jds,jde, kds,kde,                        &
           ims,ime, jms,jme, kms,kme,                        &
           its,ite, jts,jte, kts,kte                         )

    USE module_configure, only: grid_config_rec_type
	use module_state_description, only:  num_chem
	use module_data_sorgam
	use module_mixactivate, only: mixactivate

! wrapper to call mixactivate for sorgam description of aerosol

	implicit none

!   subr arguments
	integer, intent(in) ::                  &
		id, ktau,                       &
		ids, ide, jds, jde, kds, kde,   &
		ims, ime, jms, jme, kms, kme,   &
		its, ite, jts, jte, kts, kte,   &
                idrydep_onoff

	real, intent(in) :: dtstep

	real, intent(in),   &
		dimension( ims:ime, kms:kme, jms:jme ) :: &
		rho_phy, t_phy, w,   &
		z, dz8w, p_at_w, t_at_w, exch_h

	real, intent(inout),   &
		dimension( ims:ime, kms:kme, jms:jme ) :: cldfra, cldfra_old

	real, intent(in),   &
		dimension( its:ite, jts:jte, num_chem ) :: ddvel

	real, intent(in),   &
		dimension( ims:ime, kms:kme, jms:jme ) :: &
		qv, qc, qi

    LOGICAL, intent(in) :: f_qc, f_qi

	real, intent(inout),   &
		dimension( ims:ime, kms:kme, jms:jme  ) :: &
		qndrop3d

	real, intent(inout),   &
		dimension( ims:ime, kms:kme, jms:jme, 1:num_chem ) :: &
		chem
      real, intent(out), dimension(ims:ime,kms:kme,jms:jme) :: nsource, &
	     ccn1,ccn2,ccn3,ccn4,ccn5,ccn6  ! number conc of aerosols activated at supersat

	type(grid_config_rec_type), intent(in) :: config_flags

! local vars
	real qsrflx(ims:ime, jms:jme, num_chem) ! dry deposition flux of aerosol
	real sumhygro,sumvol
	integer i,j,k,l,m,n
	real hygro( its:ite, kts:kte, jts:jte,maxd_asize, maxd_atype )

! calculate volume-weighted bulk hygroscopicity for each type and size

      do 100 j=jts,jte
      do 100 k=kts,kte
      do 100 i=its,ite
       do n=1,ntype_aer
       do m=1,nsize_aer(n)
	       sumhygro=0
	       sumvol=0
	       do l=1,ncomp_aer(n)
	          sumhygro = sumhygro+hygro_aer(l,n)*   &
                   chem(i,k,j,massptr_aer(l,m,n,ai_phase))/dens_aer(l,n)
	          sumvol = sumvol+chem(i,k,j,massptr_aer(l,m,n,ai_phase))/dens_aer(l,n)
	       end do ! comp
               hygro(i,k,j,m,n)=sumhygro/sumvol
	end do ! size
	end do ! type
  100 continue


! check arguments of mixactivate for consistency between send, receive
! 06-nov-2005 rce - id & ktau added to arg list
      call mixactivate(  msectional, &
           chem, num_chem, qv, qc, qi, qndrop3d,   &
           t_phy, w, ddvel, idrydep_onoff,  &
           maxd_acomp, maxd_asize, maxd_atype, maxd_aphase,   &
           ncomp_aer, nsize_aer, ntype_aer, nphase_aer,  &
           numptr_aer, massptr_aer, dlo_sect, dhi_sect, sigmag_aer, dcen_sect,  &
           dens_aer, mw_aer,           &
           waterptr_aer, hygro,  ai_phase, cw_phase,                 &
           ids,ide, jds,jde, kds,kde,                            &
           ims,ime, jms,jme, kms,kme,                            &
           its,ite, jts,jte, kts,kte,                            &
           rho_phy, z, dz8w, p_at_w, t_at_w, exch_h,      &
           cldfra, cldfra_old, qsrflx,                      &
	       ccn1, ccn2, ccn3, ccn4, ccn5, ccn6, nsource,       &
	       id, ktau, dtstep, &
           f_qc, f_qi               )

      end subroutine sorgam_mixactivate


END MODULE module_mixactivate_wrappers
